#!/usr/bin/python
"""
Goes through all WARs and JARs and deletes JndiLookup.class files
"""

import argparse
import platform
import sys
import os
import shutil
from zipfile import ZipFile
from tempfile import NamedTemporaryFile
                            
def possibleSpecialCommand(name):
    return name.split(os.sep)[-1] in ["tsm", "tabcmd", "tsm.cmd", "tabcmd.cmd"]

def possibleZip(name):
    return name[-4:] in [".jar", ".war", ".ear"]

def removeClassesFromZip(chain, zin, log):
    found = False
    tmpfile = NamedTemporaryFile(suffix=".zip")
    with ZipFile(tmpfile, "w") as zout:
        items = zin.infolist()

        totalZips = len([i for i in items if possibleZip(i.filename)])
        zipIndex = 1
        padding = "  "*(len(chain))
        for item in items:
            # this is always a forward slash due to it being in a zip file
            if item.filename.endswith("/JndiLookup.class"):
                detected = "Detected {} in {}".format(item.filename, "->".join(chain))
                print(padding + detected)
                log.append(detected)
                found = True
            elif possibleZip(item.filename):
                print("{}[{}/{}] Scanning {}".format(padding, zipIndex, totalZips, item.filename))
                zipIndex += 1
                with NamedTemporaryFile(suffix=".zip") as f:
                    f.write(zin.read(item.filename))
                    z = ZipFile(f, "r")
                    newz, removed = removeClassesFromZip(chain + [item.filename], z, log)
                    zout.writestr(item, newz.read())
                    newz.close()

                    if removed:
                        found = True
            else:    
                zout.writestr(item, zin.read(item.filename))

    # later code will read from the tmpfile, so set it back to zero
    tmpfile.seek(0)
    return tmpfile, found

def updateInPlace(fname, dryrun, log):
    tmpfile, found = removeClassesFromZip([fname], ZipFile(fname, "r"), log)
    if found:
        if dryrun:
            print("[DRY RUN] Would have updated {}".format(fname))
        else:
            if possibleSpecialCommand(fname):
                with open(fname, "r") as oldzip:
                    script = oldzip.read().split("PK", 1)[0]

                with open(fname, "w") as f:
                    f.write(script)
                    f.write(tmpfile.read())
            else:
                with open(fname, "w") as f:
                    f.write(tmpfile.read())
            print("Updated {}".format(fname))
    tmpfile.close()

def scanAndUpdate(folder, dryrun, log):
    possibleFiles = []
    badFiles = []
    for root, dirs, files in os.walk(folder):
        for fname in files:
            fullname = os.path.join(root, fname)
            if possibleZip(fullname) or possibleSpecialCommand(fullname):
                possibleFiles.append(fullname)
            if fname == "JndiLookup.class":
                detected = "Detected {}".format(fullname)
                log.append(detected)
                if dryrun:
                    print("[DRY RUN] Would have deleted {}".format(fullname))
                else:
                    os.remove(fullname)
                    print("Deleted {}".format(fullname))

    for index, fname in enumerate(possibleFiles):
        print("[{}/{}] Scanning {}".format(index+1, len(possibleFiles), fname))
        updateInPlace(fname, dryrun, log)


if __name__ == "__main__":
    if platform.system() == "Linux":
        directories = ["/var/opt/tableau/tableau_server", "/opt/tableau/tableau_server"]
    else:
        directories = [""]

    parser = argparse.ArgumentParser(description="Goes through all WARs and JARs and deletes JndiLookup.class files")
    parser.add_argument("--dryrun", action="store_true",
                        help="Just print where the problematic files are and don't update in place")
    parser.add_argument("directories", nargs="*", default=directories,
                        help="Directories to scan")

    args = parser.parse_args()

    log = []
    for directory in args.directories:
        scanAndUpdate(directory, args.dryrun, log)

    print("==================================================================")
    print("Findings:")
    for l in log:
        print("  " + l)

